package com.tuoheng.admin.enums;

import lombok.Getter;

/**
 * @Author: 吴彬
 * @CreateTime: 2023-05-18  19:09
 * @Description: MQTT topic 枚举。将所有和MQTT交互的topic统一维护
 *      %S：设备编号占位符；qos：为传输等级定义的时候自行指定；desc：描述信息方便记录日志等
 * @Version: 1.0
 */
public enum MQTTTopicEnum {

    /**
     * 机场日志记录
     */
    TOPIC_POST_WTH("/v1/%s/log", 2, "机场日志记录");

    MQTTTopicEnum(String topic, int qos, String desc) {
        this.topic = topic;
        this.qos = qos;
        this.desc = desc;
    }

    /**
     * MQTT topic
     */
    @Getter
    private String topic;

    /**
     * 对应qos等级 0-至多一次；1-至少一次；2-确保只有一次
     */
    @Getter
    private int qos;

    /**
     * 描述
     */
    @Getter
    private String desc;

}
